/**
 * @file   arma.h
 *
 * @author X-Soldiers Team
 * 
 * @brief  Fichero de definicion del arma.
 * 
 */

#ifndef _ARMA_H_
#define _ARMA_H_

//includes

#include "constantes.h"

/**
 * @class Arma
 *
 * @brief Controla las caracteristicas y funciones del arma
 *
 */

class Arma{

	private:
		
		/** Daño que realiza el arma
		*/
		int damage_;

		/** La velocidad base de la bala
		*/
		int velb_;

		/** Rango de la bala
		*/
		int rango_;

		
	public:

		/**
   		* Constructor de un objeto de la clase Arma.
		*/
		
		Arma(){
			damage_=DAMAGE;	//DAMAGE es una constante definida para el daño del jugador.
			velb_=VELB;	//VELB es una constante definida para el valor base de la velocidad de la bala.
			rango_=RANGO; //RANGO es una constante definida para el rango que ocupa la bala.
		}


		/**
   		* Metodo que devuelve el Damage del arma.
		* @return Damage actual del arma.
   		*/
		int Damage() const{ 
			return damage_; 
		}

		/**
   		* Metodo que devuelve la velocidad actual de la bala.
		* @return velocidad actual de la bala.
   		*/
		int Velb() const{ 
			return velb_; 
		}

		/**
   		* Metodo que devuelve el rango del arma.
		* @return rango actual del arma.
   		*/

		int Rango(){
			return rango_;
		}
		
		/**
   		* Metodo que modifica el Damage del arma.
		* @param int d 		modificador del daño del arma
   		*/
		void Damage(int d){ 
			damage_+=d;
		}

		/**
   		* Metodo que modifica la velocidad actual de la bala.
		* @param int v 		modificador de la velocidad de la bala
   		*/
		void Velb(int v){ 
			velb_+=v; 
		}

		/**
   		* Metodo que modifica el rango del arma.
		* @param int r		modificador del rango de la bala
   		*/

		void Rango(int r){
			rango_=r;
		}
};
#endif
