/**
 * @file   jugador.h
 *
 * @author X-Soldiers Team
 * 
 * @brief  Fichero de definicion del primer soldado.
 * 
 */

#ifndef _JUGADOR_H_
#define _JUGADOR_H_

//includes
#include <ctime>
#include "constantes.h"
#include "arma.h"

/**
 * @class Jugador
 *
 * @brief Controla las acciones del soldado y sus caracteristicas
 *
 */

class Jugador{

	private:
		
		/** La vida que tiene el soldado
		*/
		int vida_;

		/** La velocidad base del soldado
		*/
		int velocidad_;
	
		/** Posiciones de nuestro soldado
		*/
		int posX_,posY_;


		/** Arma asociada a nuestro jugador
		*/
		Arma* arma_;

		/** Control del estado del jugador, manejado con enteros para simbolizar el estado a pintar
		*/
		int estado_;

		//este estado lo manejamos con enteros, predefinidos en un pdf adjunto al juego; nos sirve para controlar el estado y simular el movimiento


		/** Controles de tiempo
		*/
		clock_t ai,af;	//Para el control del movimiento.

	public:

		/**
   		* Constructor de un objeto de la clase Jugador.
   		* @param int px 	Posicion en el eje de las x.
   		* @param int py		Posicion en el eje de las y.
		*/
		
		Jugador(int px, int py):posX_(px),posY_(py){
			vida_=VIDA;	//VIDA es una constante definida para el valor de la vida del jugador.
			velocidad_=VELOCIDAD;	//VELOCIDAD es una constante definida para el valor base de la velocidad del jugador.
			estado_=7;
			ai=clock(); //relojes para el control del power-up, de momento NO operativos.
			af=clock();
		}


		/**
   		* Metodo que devuelve la vida actual del soldado.
		* @return vida actual del jugador.
   		*/
		int Vida() const{ 
			return vida_; 
		}

		/**
   		* Metodo que devuelve la velocidad actual del soldado.
		* @return velocidad actual del jugador.
   		*/
		int Velocidad() const{ 
			return velocidad_; 
		}

		/**
   		* Metodo que devuelve la posicion en el eje X del soldado.
		* @return posicion eje X.
   		*/

		int PosicionX() const{
			return posX_;
		}

		/**
   		* Metodo que devuelve la posicion en el eje Y del soldado.
		* @return posicion eje Y.
   		*/

		int PosicionY() const{
			return posY_;
		}

		/**
		* Metodo que devuelve el arma asociada a nuestro soldado.
		* @return arma del jugador.
		*/

		Arma* ArmaActual() const{
			return arma_;
		}

		/**
   		* Metodo que modifica la vida actual del soldado.
		* @param int vida	modificador de la vida del jugador
   		*/
		void Vida(int vida){ 
			vida_-=vida; 
		}

		/**
   		* Metodo que modifica la velocidad actual del soldado.
		* @param int v 		modificador de la velocidad del personaje
   		*/
		void Velocidad(int v){ 
			velocidad_+=v; 
		}

		/**
   		* Metodo que modifica la posicion en el eje X del soldado.
		* @param int posx 	modificador de la posición x del personaje en el mapa
   		*/

		void PosicionX(int posx){
			posX_=posx;
		}

		/**
   		* Metodo que modifica la posicion en el eje Y del soldado.
		* @param int posy 	modificador de la posición y del personaje en el mapa
   		*/

		void PosicionY(int posy){
			posY_=posy;
		}

		/**
   		* Metodo que devuelve el estado del soldado.
		* @return devuelve estado del soldado
   		*/

		int estado()const{
			return estado_;
		}

		/**
   		* Metodo que modifica el estado del soldado.
		* @param int e 		modificador del estado en el que se encuentra el personaje
   		*/

		void estado(int e){
			estado_=e;
		}

		/**
   		* Metodo que devuelve el estado del reloj inicial.
		* @return tiempo inicial
   		*/
		
		clock_t tiempoi()const{
			return ai;
		}
		
		/**
   		* Metodo que devuelve el estado del reloj final.
		* @return tiempo final
   		*/
		
		clock_t tiempof()const{
			return af;
		}

		/**
   		* Metodo que devuelve la diferencia del tiempo entre relojes.
		* @return diferencia del tiempo
   		*/
		
		double tiempo()const{
			return difftime(af,ai);
		}

		/**
   		* Metodo que modifica el estado del reloj inicial.
   		*/
		
		void tiempoim(){
			ai=clock();
		}

		/**
   		* Metodo que modifica el estado del reloj final.
   		*/
		
		void tiempofm(){
			af=clock();
		}
	
};
#endif

