/**
 * @file   ventana.h
 *
 * @author X-Soldiers Team
 * 
 * @brief  Fichero de definicion de la clase Ventana.
 * 
 */

#ifndef _VENTANA_H_
#define _VENTANA_H_

#include <Gosu/AutoLink.hpp>
#include <Gosu/Gosu.hpp>

#include "caja.h"
#include "constantes.h"
#include "jugador.h"
#include "arma.h"


/**
 * @class Ventana
 *
 * @brief La ventana de nuestro juego
 *
 * Implementa la clase abstracta Gosu::Window.
 *
 */

class Ventana : public Gosu::Window{
public:

    /// Constructor 
    Ventana();

    /// Destructor. Libera los recursos.
    ~Ventana();

    /// Fase de la lógica. Hace los cálculos.
    void update();

    /// Fase de dibujado. Se pintan los gráficos.
    void draw();

    /// Gestión de la entrada. Son llamados cuando se pulsa una tecla.
    void buttonDown(Gosu::Button B);

	/// Metodo para ocultar el cursor.
	bool needsCursor() const;
	///Metodo para calcular si colisiona una caja con cualquiera de las cajas del mapa (paredes)
	bool colisiona(Caja c);

private:
    /// Imágenes para cada elemento
    Gosu::Image * mapa,*mapa2,* menu,* opciones,*mcreditos,*mwinner,* bala,* bala2, * soldado1[3][4], * soldado2[3][4];
    /// Fuente del juego
    Gosu::Font * fuente, * fuentecred, *fuentevida;

	/// Sonidos y musica del juego
	Gosu::Song * musicajuego, * musicamenu, *musicawinner;

    /// Cajas de colisiones para la bala, P1, P2 y paredes del mapa
    Caja cajaP1, cajaP2, cajaBala, cajaBala2, cajaM1, cajaM2,cajaM3, cajaM4, cajaM5, cajaM6,cajaM7, cajaM8, cajaM9, cajaM10,cajaM11, cajaM12, cajaM13, cajaM14,cajaM15, cajaM16,cajaM17, cajaM18, cajaM19,cajaM20, cajaM21, cajaM22,cajaM23, cajaM24, cajaM25, cajaM26,cajaM27, cajaM28, cajaM29, cajaM30,cajaM31, cajaM32, cajaM33, cajaM34,cajaM35, cajaM36,cajaM37, cajaM38, cajaM39;

    /// Posibles estados del juego
    enum tipoEstado{ ESTADO_MENU, ESTADO_JUEGO, ESTADO_CREDITOS, ESTADO_OPCIONES, ESTADO_REINICIAR, ESTADO_WINER, ESTADO_DISPARO };

    /// Estado del juego
    tipoEstado estadoActual;
    
     /// Velocidad horizontal de la bala
    float balaVelX;
    float balaVel2X;
    /// Velocidad vertical de la bala
    float balaVelY;
    float balaVel2Y;
	/// Posicion del cursor en ese momento
	
	float mouseX, mouseY;

   ///Jugadores
	Jugador *Jugador1;
	Jugador *Jugador2;

    /// Función que reinicia la partida
    void ReiniciarPartida();   //funcion tenemos que modificarla para que sea capaz de reiniciar una partida si el usuario lo pide

};

#endif /* _VENTANA_H_ */
